// Time-stamp: "10.08.11 00:39 testfixpoint_debug.cpp klaus?wachtler.de"
//
// Tests Anyware::fixpoint<> with DEBUG_... options
//
// On Linux to use fixpoint calculations:
// g++ -Wall -DDEBUG_FIXPOINT -DDEBUG_FIXPOINT_FOLLOW_DOUBLE -DDEBUG_FIXPOINT_CHECK_RANGE -I. testfixpoint_debug.cpp -o testfixpoint_debug_fixpoint
//
// On Linux to use floating point calculations:
// g++ -Wall testfixpoint_debug.cpp -o testfixpoint_debug_float





#include <iostream>
#include <iomanip>


#include <cmath>
#include <limits>


#ifdef DEBUG_FIXPOINT

#include <fixpoint.h>

typedef    Anyware::fixpoint< int16_t, int32_t, 8, true > myType;

// include generated source for sin()
// generate it with
// g++ -Wall -Wextra generate_sin_table.cpp -o generate_sin_table
// ./generate_sin_table int16_t int32_t 16 true 0.1 fixpoint_sin_specialized_int32_t_int32_t_16_true_0.1.cpp.inc
#include "fixpoint_sin_specialized_int16_t_int32_t_8_true_0.1.cpp.inc"


#else // double

typedef    double myType;

#endif /* ifdef DEBUG_FIXPOINT .. else ... */

#ifdef DEBUG_FIXPOINT_CHECK_RANGE

// initialize min and max value for all calculations with myType
template<> double   myType::checkRangeMin = std::numeric_limits<double>::max();
template<> double   myType::checkRangeMax = std::numeric_limits<double>::min();

#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */



void testSimpleThings()
{
  std::cout << std::endl << __FUNCTION__ << std::endl;

  myType  a( 6.125 ), b( 2.5 );
  myType  c;

#ifdef DEBUG_FIXPOINT
  a.setName( "a" );
  b.setName( "b" );
  c.setName( "c" );
#endif /* ifdef DEBUG_FIXPOINT */


  c = a/b;
  std::cout << "c = a/b;\na=" << a << "\nb=" << b << "\nc=" << c << std::endl << std::endl;

#ifdef DEBUG_FIXPOINT
  c = a%b;
#else // double
  c = std::fmod( a, b );
#endif /* ifdef DEBUG_FIXPOINT */

  std::cout << "c = a%b;\na=" << a << "\nb=" << b << "\nc=" << c << std::endl << std::endl;

  c *= myType( 1.1 );
  std::cout << "c *= 1.1;\na=" << a << "\nb=" << b << "\nc=" << c << std::endl << std::endl;
}

void testSin()
{
  //std::cout << std::endl << __FUNCTION__ << std::endl;

  for( myType angle=0.0; angle<=myType(2.0*M_PI+0.01); angle+=0.5 )
  {
    //std::cout << "sin( " << angle << " ) = " << sin( angle ) << std::endl;
    std::cout << (double)angle << " " << (double)sin( angle ) << std::endl;
  }
}

void testNumericalLimits()
{
  std::cout << std::endl << __FUNCTION__ << std::endl;

  std::cout << "denorm_min(): " << std::numeric_limits<myType>::denorm_min() << std::endl;
  std::cout << "min():        " << std::numeric_limits<myType>::min() << std::endl;
  std::cout << "max():        " << std::numeric_limits<myType>::max() << std::endl;
}




int main( int nargs, char **args )
{
  //testSimpleThings();
  testSin();
  //testNumericalLimits();


#ifdef DEBUG_FIXPOINT_CHECK_RANGE

// min and max value for all calculations with myType

  std::cout << std::endl;
  std::cout << "overall min value = " << myType::checkRangeMin
            << " ( limit is " << (double)myType::checkRangeMinLimit() << " )"
            << std::endl;
  std::cout << "overall max value = " << myType::checkRangeMax
            << " ( limit is " << (double)myType::checkRangeMaxLimit() << " )"
            << std::endl;

#endif /* ifdef DEBUG_FIXPOINT_CHECK_RANGE */

}
